@@ -65,7 +65,7 @@ class ItemType(str, Enum):
         elif len(item_hash) == 64:
             return cls.storage
         else:
-            raise UnknownHashError(f"Unknown hash {len(item_hash)} {item_hash}")
+            raise UnknownHashError(f"Could not determine hash type: '{item_hash}'")
 
     @classmethod
     def is_storage(cls, item_hash: str):
@@ -76,6 +76,38 @@ class ItemType(str, Enum):
         return cls.from_hash(item_hash) == cls.ipfs
 
 
+class ItemHash(str):
+    item_type: ItemType
+
+    # When overriding str, override __new__ instead of __init__.
+    def __new__(cls, value, item_type: ItemType):
+        obj = str.__new__(cls, value)
+        obj.item_type = item_type
+        return obj
+
+    @classmethod
+    def __get_validators__(cls):
+        # one or more validators may be yielded which will be called in the
+        # order to validate the input, each validator will receive as an input
+        # the value returned from the previous validator
+        yield cls.validate
+
+    @classmethod
+    def validate(cls, v):
+        if not isinstance(v, str):
+            raise TypeError("Item hash must be a string")
+
+        try:
+            item_type = ItemType.from_hash(v)
+        except UnknownHashError as e:
+            raise ValueError(str(e))
+
+        return cls(v, item_type)
+
+    def __repr__(self):
+        return f"<ItemHash value={super().__repr__()} item_type={self.item_type!r}>"
+
+
 class MongodbId(BaseModel):
     """PyAleph returns an internal MongoDB id"""
 
@@ -91,7 +123,7 @@ class ChainRef(BaseModel):
     chain: Chain
     channel: Optional[str] = None
     item_content: str
-    item_hash: str
+    item_hash: ItemHash
     item_type: ItemType
     sender: str
     signature: str
@@ -167,7 +199,7 @@ class StoreContent(BaseContent):
     """Content of a STORE message"""
 
     item_type: ItemType
-    item_hash: str
+    item_hash: ItemHash
     size: Optional[int] = None  # Generated by the node on storage
     content_type: Optional[str] = None  # Generated by the node on storage
     ref: Optional[str] = None
@@ -179,8 +211,8 @@ class StoreContent(BaseContent):
 class ForgetContent(BaseContent):
     """Content of a FORGET message"""
 
-    hashes: List[str]
-    aggregates: List[str] = Field(default_factory=list)
+    hashes: List[ItemHash]
+    aggregates: List[ItemHash] = Field(default_factory=list)
     reason: Optional[str] = None
 
     def __hash__(self):
@@ -226,7 +258,7 @@ class BaseMessage(BaseModel):
     hash_type: Optional[HashType] = Field(
         default=None, description="Hashing algorithm used to compute 'item_hash'"
     )
-    item_hash: str = Field(description="Hash of the content (sha256 by default)")
+    item_hash: ItemHash = Field(description="Hash of the content (sha256 by default)")
     content: BaseContent = Field(description="Content of the message, ready to be used")
 
     forgotten_by: Optional[List[str]]
