@@ -1,14 +1,16 @@
 from __future__ import annotations
 
 from abc import ABC, abstractmethod
-
 from enum import Enum
+from typing import Optional, List, Union, Dict, Any, NewType
+
 from pydantic import Field, Extra, conint
-from typing import Optional, List, Union, Dict, Any
 from typing_extensions import Literal
 
 from .abstract import BaseContent, HashableModel
 
+Megabytes = NewType("Megabytes", int)
+
 
 class Encoding(str, Enum):
     plain = "plain"
@@ -47,6 +49,10 @@ class Subscription(HashableModel):
 class FunctionTriggers(HashableModel):
     http: bool
     message: Optional[List[Subscription]] = None
+    persistent: Optional[bool] = None
+
+    class Config:
+        extra = Extra.forbid
 
 
 class FunctionEnvironment(HashableModel):
@@ -62,6 +68,43 @@ class MachineResources(HashableModel):
     seconds: int = 1
 
 
+class CpuProperties(HashableModel):
+    """CPU properties."""
+
+    architecture: Optional[Literal["x86_64", "arm64"]] = Field(
+        default=None, description="CPU architecture"
+    )
+    vendor: Optional[
+        Union[Literal["AuthenticAMD", "GenuineIntel"], str]
+    ] = Field(default=None, description="CPU vendor. Allows other vendors.")
+
+    class Config:
+        extra = Extra.forbid
+
+
+class NodeRequirements(HashableModel):
+    owner: Optional[str] = Field(default=None, description="Address of the node owner")
+    address_regex: Optional[str] = Field(
+        default=None, description="Node address must match this regular expression"
+    )
+
+    class Config:
+        extra = Extra.forbid
+
+
+class HostRequirements(HashableModel):
+    cpu: Optional[CpuProperties] = Field(
+        default=None, description="Required CPU properties"
+    )
+    node: Optional[NodeRequirements] = Field(
+        default=None, description="Required Compute Resource Node properties"
+    )
+
+    class Config:
+        # Allow users to add custom requirements
+        extra = Extra.allow
+
+
 class FunctionRuntime(HashableModel):
     ref: str
     use_latest: bool = True
@@ -73,7 +116,8 @@ class AbstractVolume(HashableModel, ABC):
     mount: Optional[str] = None
 
     @abstractmethod
-    def is_read_only(self): ...
+    def is_read_only(self):
+        ...
 
     class Config:
         extra = Extra.forbid
@@ -117,20 +161,30 @@ class ProgramContent(HashableModel, BaseContent):
     allow_amend: bool = Field(description="Allow amends to update this function")
     code: CodeContent = Field(description="Code to execute")
     metadata: Optional[Dict[str, Any]] = Field(description="Metadata of the VM")
-    variables: Optional[Dict[str, str]] = Field(default=None, description="Environment variables available in the VM")
-    data: Optional[DataContent] = Field(default=None, description="Data to use during computation")
-    export: Optional[Export] = Field(default=None, description="Data to export after computation")
+    variables: Optional[Dict[str, str]] = Field(
+        default=None, description="Environment variables available in the VM"
+    )
+    data: Optional[DataContent] = Field(
+        default=None, description="Data to use during computation"
+    )
+    export: Optional[Export] = Field(
+        default=None, description="Data to export after computation"
+    )
     on: FunctionTriggers = Field(description="Signals that trigger an execution")
-    environment: FunctionEnvironment = Field(description="Properties of the execution environment")
+    environment: FunctionEnvironment = Field(
+        description="Properties of the execution environment"
+    )
     resources: MachineResources = Field(description="System resources required")
+    requirements: Optional[HostRequirements] = Field(
+        default=None, description="System properties required"
+    )
     runtime: FunctionRuntime = Field(
         description="Execution runtime (rootfs with Python interpreter)"
     )
     volumes: List[MachineVolume] = Field(
-        default=[],
-        description="Volumes to mount on the filesystem"
+        default=[], description="Volumes to mount on the filesystem"
     )
     replaces: Optional[str] = Field(
         default=None,
-        description="Previous version to replace. Must be signed by the same address"
+        description="Previous version to replace. Must be signed by the same address",
     )
